<?php
$requestUri = $_SERVER['REQUEST_URI'];

if ($requestUri === '/logo.png') return false;

if ($requestUri !== '/') {
    header("HTTP/1.0 404 Not Found");
    echo "404 Not Found";
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {

    if (!isset($_FILES['files']))
    {
        http_response_code(400);
        echo "无效 POST 参数\n";
        exit;
    }

    $uploadDir = __DIR__ . '/uploads/';

    if (!is_dir($uploadDir)) {
        if (!mkdir($uploadDir, 0777, true)) {
            http_response_code(500);
            echo "服务器权限错误\n";
            exit;
        }
    }

    $clientIP = $_SERVER['REMOTE_ADDR'];

    foreach ($_FILES['files']['name'] as $key => $name) {
        $tmpName = $_FILES['files']['tmp_name'][$key];
        
        if (!isset($_POST['relative_paths'][$key])) {
            http_response_code(400);
            echo "缺失文件路径 - $name\n";
            continue;
        }

        $relativePath = $_POST['relative_paths'][$key];
        $filePath = $uploadDir . $relativePath;
        $firstLevelDir = explode('/', $relativePath)[0];
        $fullName = $relativePath;

        if ($firstLevelDir === $relativePath) {
            http_response_code(400);
            echo "文件路径非法 - $relativePath\n";
            continue;
        }

        if (file_exists($filePath)) {
            if (hash_file('md5', $tmpName) === hash_file('md5', $filePath)) {
                echo "跳过相同文件 - $fullName\n";
                continue;
            }

            $firstLevelDir .= ' + ' . $clientIP;
            $relativePath = $firstLevelDir . '/' . implode('/', array_slice(explode('/', $relativePath), 1));
            $filePath = $uploadDir . $name;
        }

        $fileDir = dirname($filePath);
        if (!is_dir($fileDir)) {
            if (!mkdir($fileDir, 0777, true)) {
                http_response_code(500);
                echo "无法创建路径 - $fullName\n";
                exit;
            }
        }

        if (!move_uploaded_file($tmpName, $filePath)) {
            http_response_code(500);
            echo "无法写入文件 - $fullName\n";
            exit;
        }

        echo "文件上传完成 - $fullName\n";
    }

    exit;
}
?>

<!DOCTYPE html>
<html lang="zh-CN">
<head>
    <meta charset="UTF-8">
    <title>提交答案</title>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style>
        body {
			font-family: Arial, sans-serif;
			display: flex;
			justify-content: center;
			align-items: center;
			height: 80vh;
			margin: 0;
			background-color: #f0f0f0;
			flex-direction: column;
        }
		.container {
			width: 90%;
			max-width: 600px;
		}
		.logo {
            display: block;
            margin: 0 auto;
            width: 80%;
            margin-bottom: 15px;
		}
		.upload-container {
			display: flex;
			flex-direction: column;
			gap: 10px;
			padding: 20px;
			border-radius: 10px;
			box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);
			background-color: #fff;
		}
        .drop-zone {
            border: 2px dashed #007bff;
            border-radius: 10px;
            padding: 20px;
            text-align: center;
            color: #007bff;
            cursor: pointer;
            transition: background-color 0.3s ease;
			margin-bottom: 15px;
        }
        .drop-zone:hover {
            background-color: #f0f8ff;
        }
        .drop-zone p {
            margin: 0;
        }
        input, select {
            width: 100%;
            padding: 10px;
            margin: 10px 0;
        }
		button {
			padding: 10px;
			background-color: #007bff;
			color: white;
			border: none;
			border-radius: 5px;
			cursor: pointer;
			width: 100%;
		}
		button:hover {
			background-color: #0056b3;
		}
    </style>
</head>

<body>
    <div class="container">
        <img src="logo.png" alt="Logo" class="logo">
            <div class="upload-container">
                <form id="uploadForm" method="post" enctype="multipart/form-data">
                    <div id="dropZone" class="drop-zone">
                        <p>点击此处或拖放以选择考试文件夹</p>
                        <input type="file" id="fileInput" name="files[]" webkitdirectory multiple style="display: none;">
                    </div>
                    <button type="button" onclick="uploadFiles()">提交答案</button>
                </form>
            </div>
        </div>
    </div>
    <footer style="position: fixed; bottom: 0; right: 40px; padding: 10px; color: #606060; text-align: right;">
        <p>By _Redstone_c_</p>
    </footer>

    <script>
        const form      = document.getElementById('uploadForm');
        const dropZone  = document.getElementById('dropZone');
        const fileInput = document.getElementById('fileInput');

        fileInput.addEventListener('change', updateFileList);

        dropZone.addEventListener('click', () => fileInput.click());

        dropZone.addEventListener('dragover', (e) => {
            e.preventDefault();
            dropZone.style.backgroundColor = '#f0f8ff';
        });

        dropZone.addEventListener('dragleave', () => {
            dropZone.style.backgroundColor = '';
        });

        dropZone.addEventListener('drop', async (e) => {
            e.preventDefault();
            dropZone.style.backgroundColor = '';

            const items = e.dataTransfer.items;
            const files = [];

            async function traverseFileTree(item, path = "") {
                if (item.isFile) {
                    const file = await new Promise((resolve) => item.file(resolve));
                    file.relativePath = path + file.name;
                    files.push(file);
                } else if (item.isDirectory) {
                    const dirReader = item.createReader();
                    const entries = await new Promise((resolve) => dirReader.readEntries(resolve));
                    await Promise.all(entries.map(entry => traverseFileTree(entry, path + item.name + "/")));
                }
            }

            if (items.length !== 1) {
                alert('请拖放单个文件夹！');
                return;
            }

            const entry = items[0].webkitGetAsEntry();
            if (!entry.isDirectory) {
                alert('请拖放单个文件夹！');
                return;
            }
            
            await traverseFileTree(entry);

            const dataTransfer = new DataTransfer();
            files.forEach(file => dataTransfer.items.add(file));
            fileInput.files = dataTransfer.files;

            updateFileList();
        });
        
        function updateFileList() {
            const fileList = document.createElement('ul');
            fileList.style.listStyleType = 'none';
            fileList.style.padding = '0';

            if (fileInput.files.length === 0) {
                dropZone.innerHTML = '<p>点击此处或拖放以选择考试文件夹</p>';
                return;
            }

            for (let i = 0; i < fileInput.files.length; i++) {
                const file = fileInput.files[i];
                const listItem = document.createElement('li');
                listItem.textContent = file.relativePath || file.webkitRelativePath || file.name;
                fileList.appendChild(listItem);
            }

            dropZone.innerHTML = '';
            dropZone.appendChild(fileList);
        }

        function uploadFiles() {
            const formData = new FormData();

            for (let i = 0; i < fileInput.files.length; i++) {
                const file = fileInput.files[i];
                formData.append('files[]', file);
                formData.append('relative_paths[]', file.relativePath || file.webkitRelativePath);
            }

            console.log('Uploading files...');

            fetch(form.action, {
                method: 'POST',
                body: formData
            }).then(response => {
                if (response.ok) {
                    console.log('File upload successfully!');
                    return response.text().then(data => {
                        alert(data);
                        fileInput.value = '';
                        updateFileList();
                    });
                } else {
                    console.log('File upload failed!');
                    return response.text().then(data => {
                        alert(data);
                    });
                }
            }).catch(error => {
                console.error('Error:', error);
                alert('前端内部错误！');
            });
        }
    </script>
</body>

</html>
